@@ -2,7 +2,7 @@
 // All Rights Reserved.
 // This code is published under the Eclipse Public License.
 //
-// $Id: IpIpoptApplication.cpp 2346 2013-07-01 13:03:35Z stefan $
+// $Id: IpIpoptApplication.cpp 2443 2013-11-26 11:34:33Z stefan $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-09-02
 
@@ -59,6 +59,7 @@ namespace Ipopt
                                      bool create_empty /* = false */)
       :
       read_params_dat_(true),
+      rethrow_nonipoptexception_(false),
       inexact_algorithm_(false),
       replace_bounds_(false)
   {
@@ -113,6 +114,7 @@ namespace Ipopt
                                      SmartPtr<Journalist> jnlst)
       :
       read_params_dat_(true),
+      rethrow_nonipoptexception_(false),
       jnlst_(jnlst),
       reg_options_(reg_options),
       options_(options),
@@ -439,9 +441,11 @@ namespace Ipopt
 #if defined(HAVE_PARDISO) || defined(HAVE_LINEARSOLVERLOADER)
 
           options_to_print.push_back("#Pardiso Linear Solver");
-          options_to_print.push_back("pardiso_msglvl");
           options_to_print.push_back("pardiso_matching_strategy");
-          options_to_print.push_back("pardiso_out_of_core_power");
+          options_to_print.push_back("pardiso_max_iterative_refinement_steps");
+          options_to_print.push_back("pardiso_msglvl");
+          options_to_print.push_back("pardiso_order");
+          //options_to_print.push_back("pardiso_out_of_core_power");
 #endif
 
 #ifdef HAVE_WSMP
@@ -516,9 +520,16 @@ namespace Ipopt
       return Insufficient_Memory;
     }
     catch (...) {
-      IpoptException exc("Unknown Exception caught in ipopt", "Unknown File", -1);
-      exc.ReportException(*jnlst_);
-      return NonIpopt_Exception_Thrown;
+      if( !rethrow_nonipoptexception_ )
+      {
+        IpoptException exc("Unknown Exception caught in ipopt", "Unknown File", -1);
+        exc.ReportException(*jnlst_);
+        return NonIpopt_Exception_Thrown;
+      }
+      else
+      {
+        throw;
+      }
     }
     return Solve_Succeeded;
   }
@@ -536,9 +547,16 @@ namespace Ipopt
         return Insufficient_Memory;
       }
       catch (...) {
-        IpoptException exc("Unknown Exception caught in ipopt", "Unknown File", -1);
-        exc.ReportException(*jnlst_);
-        return NonIpopt_Exception_Thrown;
+        if( !rethrow_nonipoptexception_ )
+        {
+          IpoptException exc("Unknown Exception caught in ipopt", "Unknown File", -1);
+          exc.ReportException(*jnlst_);
+          return NonIpopt_Exception_Thrown;
+        }
+        else
+        {
+          throw;
+        }
       }
     }
     ApplicationReturnStatus retval = Initialize(is);
@@ -765,9 +783,16 @@ namespace Ipopt
       jnlst_->Printf(J_SUMMARY, J_MAIN, "\nEXIT: Not enough memory.\n");
     }
     catch (...) {
-      IpoptException exc("Unknown Exception caught in Ipopt", "Unknown File", -1);
-      exc.ReportException(*jnlst_, J_ERROR);
-      retValue = NonIpopt_Exception_Thrown;
+      if( !rethrow_nonipoptexception_ )
+      {
+        IpoptException exc("Unknown Exception caught in Ipopt", "Unknown File", -1);
+        exc.ReportException(*jnlst_, J_ERROR);
+        retValue = NonIpopt_Exception_Thrown;
+      }
+      else
+      {
+        throw;
+      }
     }
 
     jnlst_->FlushBuffer();
@@ -1047,9 +1072,17 @@ namespace Ipopt
       status = OUT_OF_MEMORY;
     }
     catch (...) {
-      IpoptException exc("Unknown Exception caught in Ipopt", "Unknown File", -1);
-      exc.ReportException(*jnlst_, J_ERROR);
-      retValue = NonIpopt_Exception_Thrown;
+      if( !rethrow_nonipoptexception_ )
+      {
+        IpoptException exc("Unknown Exception caught in Ipopt", "Unknown File", -1);
+        exc.ReportException(*jnlst_, J_ERROR);
+        retValue = NonIpopt_Exception_Thrown;
+      }
+      else
+      {
+        jnlst_->FlushBuffer();
+        throw;
+      }
     }
 
     if (!skip_finalize_solution_call)
